# coding=utf-8
import datetime
from tfduck.bdp_sdk_py.config.bdpmanager import BdpManager
mlog = print


ISTEST = True  # 为True就是测试模式
BDPTOKEN = 'your token'  # 需要自己填写bdp_token，找相关人员索要
MAX_DATA_LENGTH = 5000
# 定义上传的表的结构
PUBLIC_ACTIVATE_HEADER = (
    ("app_name", "string"),
    ("date", "date"),
    ("country", "string"),
    ("active_count", "number"),
)

PUBLIC_ACTIVATE_HEADER1 = (
    ("app_name", "string"),
    ("date", "date"),
    ("country", "string"),
    ("active_count", "number"),
)

# 定义表名
HEADERDEFINE = {
    "af_active": PUBLIC_ACTIVATE_HEADER,
    "af_active1": PUBLIC_ACTIVATE_HEADER1,
}
# 定义表和表结构映射关系
BDPTABDEFINE = [
    {
        'headers': HEADERDEFINE['af_active'],
        'bdpTbName': "af_active",
    },
    {
        'headers': HEADERDEFINE['af_active1'],
        'bdpTbName': "af_active1",
    },
]


def sync():
    """
    @des: 同步数据
    """
    # 我们进行一个完整的同步到af_active表的操作
    table_name = "af_active"
    # 获取bdp操作实例
    bdpmananger = BdpManager.getInstance(
        BDPTOKEN, "openAPI", BDPTABDEFINE, ISTEST, MAX_DATA_LENGTH)
    # revert 需要操作的表
    bdpmananger.revertTable(table_name)
    # 删除需要更新的数据--避免重复
    dateQuery = {
        "dateUniqKey": "date",
        "begin": "2020-03-25",
        "end": "2020-03-26"
    }
    result = bdpmananger.bulkdeleteTable(
        "af_active", dateQuery, None)
    if not result:
        raise("delete error")
    # 插入数据
    _datas = [["app1", "2020-03-25", "us", 10],
              ["app2", "2020-03-25", "CA", 15]]
    bdpmananger.insertData(table_name, _datas)
    # commit --- commit有时间限制，1个小时只能不能重复commit(可以删除测试表，进行重复commit)
    bdpmananger.commitTable(table_name)
    # update 到 关联表
    bdpmananger.updateTable([table_name])
    # 结束
    print("sync end")


def view_table():
    """
    @des: 预览bdp表里的数据
    """
    table_name = "af_active"
    # 获取bdp操作实例
    bdpmananger = BdpManager.getInstance(
        BDPTOKEN, "openAPI", BDPTABDEFINE, ISTEST, MAX_DATA_LENGTH)
    table = bdpmananger.getTable(table_name)
    data = table.preview()
    data = data["data"]
    data.sort(key=lambda x: x[1])
    mlog(len(data))
    mlog(data)
    # mlog([d for d in data][-10:])
    # mlog([d for d in data][0:10])
    print("view end")


def delete_table():
    """
    @des: 删除bdp的表--小心操作
    """
    table_name = "af_active"
    # 获取bdp操作实例
    bdpmananger = BdpManager.getInstance(
        BDPTOKEN, "openAPI", BDPTABDEFINE, ISTEST, MAX_DATA_LENGTH)
    bdpmananger.deleteTable(table_name)
    print("delete end")


if __name__ == "__main__":
    sync()
    # view_table()
    # delete_table()



"""
from tfduck.tinkdata.query import Query
tq = Query("token", "tga_query_ip", "tga_query_port")
datas = tq.query(sql="select to_char("#event_time", 'yyyymmdd') as date, "#user_id","#event_name", "#app_id", "#country", "#country_code" from v_event_15 where "$part_event"='click_ad' and "$part_date">'2020-04-01' and "$part_date"<'2020-04-03' limit 100")
"""
import requests


class Query(object):
    def __init__(self, token, query_ip, query_port):
        self.query_uri = "http://da.163py.com:8889/get_result"
        self.token = token
        self.query_ip = query_ip
        self.query_port = query_port

    def query(self, sql):
        """
        @des: query from thinkdata
        @param sql: select to_char("#event_time", 'yyyymmdd') as date, "#user_id","#event_name", "#app_id", "#country", "#country_code" from v_event_15 where "$part_event"='click_ad' and "$part_date">'2020-04-01' and "$part_date"<'2020-04-03' limit 100
        @return: {"header":[数据头], "datas":[数据列表]}
        """
        res = requests.post(self.query_uri, data={
                            "sql": sql, "token": self.token, "query_ip": self.query_ip, "query_port": self.query_port}, timeout=(60, 600))
        query_data = res.json()
        #
        datas = query_data['result']['datas']
        return datas




# coding=utf-8
"""
@des: bdp管理器
bdpmananger = BdpManager.getInstance(BDPTOKEN, "openAPI", BDPTABDEFINE, ISTEST, MAX_DATA_LENGTH)
"""
from tfduck.bdp_sdk_py.opends.opends import OpenDSException
from tfduck.bdp_sdk_py.opends.sdk import BDPClient


def mmlog(*args, **kwargs):
    try:
        if args:
            print(args)
        if kwargs:
            print(kwargs)
    except:
        pass


mlog = mmlog


def getbdpOutputDefine(BDPTABDEFINE):
    bdpOutputDefine = {}
    for obj in BDPTABDEFINE:
        bdpOutputDefine[obj['bdpTbName']] = {
            "headers": obj['headers']
        }
        bdpOutputDefine["test_%s" % obj['bdpTbName']] = {
            "headers": obj['headers']
        }
    return bdpOutputDefine


class BdpManager(object):
    """
    @des
    """

    def __init__(self, bdp_token, bdp_rd, BDPTABDEFINE, ISTEST, MAX_DATA_LENGTH):
        self.client = None
        self.dsData = None
        self.outputHeaders = None
        self.ISTEST = ISTEST
        self.MAX_DATA_LENGTH = MAX_DATA_LENGTH
        self.init(bdp_token, bdp_rd, BDPTABDEFINE)

    @classmethod
    def getInstance(cls, *args, **kwargs):
        return BdpManager(*args, **kwargs)

    def init(self, bdp_token, bdp_rd, BDPTABDEFINE):
        self.outputHeaders = getbdpOutputDefine(BDPTABDEFINE)
        self.client = BDPClient(bdp_token, "*")
        mlog("all ds------------------", self.client.get_all_ds())
        try:
            self.dsData = self.client.get_ds(bdp_rd)
            mlog("bdp init completed")
        except Exception as e:
            mlog("bdp init failed", e)

    def updateTable(self, tableNames):
        """
        @des:触发级联更新，使对该工作表数据进行的操作，对其他由该工作表生成的其他工作表生效。支持批量操作，可同时更新多张工作表。
        @param tableNames 需要被刷新的工作表队列
        """
        tableIds = []
        for tableName in tableNames:
            tableIds.append(self.getTableId(tableName))
        mlog("update TableNames = ", tableNames)
        mlog("update TableIds = ", tableIds)
        result = self.dsData.update(tableIds)
        return result

    def revertTable(self, tableName):
        """
        @des:回滚工作表未提交数据
        将工作表数据回滚到上一次commit的版本。
        @param tableName 工作表名称
        """
        table = self.getTable(tableName)
        result = table.revert()
        mlog("revert table = ", tableName)
        return result

    def insertData(self, tableName, data):
        """
        @des:数据插入的接口
        写入数据须与对应字段一致，包括字段类型和数量；
        数据超过5000行，会做分段发送处理。
        @param tableName 工作表的名称
        @param data 插入的数据格式，数据结构为二维数组 '[["2016-01-12 12:00:22","marry","1"]]'
        """
        MAX_DATA_LENGTH = self.MAX_DATA_LENGTH
        if not isinstance(data, list):
            raise Exception(u"%s需要发送的数据不是数组形式" % tableName)
        dataLength = len(data)
        table = self.getTable(tableName)
        fields = [v[0] for v in self.outputHeaders[tableName]["headers"]]
        if dataLength < 1:
            mlog(u"%s没有数据需要发送" % tableName)
        elif dataLength > MAX_DATA_LENGTH:
            send_count = int(len(data) / MAX_DATA_LENGTH) + 1
            mlog(tableName, u"的数据共,", dataLength,
                 u"超过", MAX_DATA_LENGTH, u"条数据,被拆分成", send_count, u"条来向BDP发送")
            for i in range(send_count):
                _start = i * MAX_DATA_LENGTH
                _end = (i+1) * MAX_DATA_LENGTH
                real_data = data[_start:_end]
                # mlog(tableName, u"分批发送，数据长度 %s"%len(real_data))
                if real_data:
                    table.insert_data_by_name(fields, data[_start:_end])
        else:
            mlog(tableName, u"发送的数据长度", dataLength)
            table.insert_data_by_name(fields, data)

    def commitTable(self, tableName):
        """
        @des:提交工作表的数据
        最近写入数据的操作被保存在缓存文件中，调用该接口可以使操作生效。
        @param tableName 工作表名称
        """
        table = self.getTable(tableName)
        mlog("commit tableName start = ", tableName)
        result = table.commit()
        return result

    def bulkdeleteTable(self, tableName, dateQuery, otherQuery=None):
        """
        @des:
        条件删除
        根据条件删除工作表中数据。
        删除操作将匹配所有符合条件的数据后进行删除，性能不如直接删除，如是根据主键值删除时建议采用直接删除。支持SQL；
        @param tableName 工作表名称
        @param dateQuery BdpQueryDate数据结构，传入开始日期和结束日期，以及索引表头
        @params: dateQuery--{
                              "dateUniqKey": string; // 唯一索引key 'date'
                              "begin": string; // 开始时间"YYYY-MM-DD"
                              "end?": string; // 结束时间"YYYY-MM-DD""
                            }
        """
        table = self.getTable(tableName)
        where = self.createDateWhereQuery(dateQuery)
        if otherQuery is not None:
            where = "%s AND %s" % (where, otherQuery)
        mlog(tableName, "createDateWhereQuery= ", where)
        result = table.bulk_delete_yx(where)
        mlog(tableName, "bulkdeleteTable: ", result, where)
        if result[0] == "success":
            return True
        else:
            return False

    def createDateWhereQuery(self, dateQuery):
        where1 = """ `%s` >= '%s 00:00:00' """ % (
            dateQuery['dateUniqKey'], dateQuery['begin'])
        where2 = """ `%s` < '%s 00:00:00' """ % (
            dateQuery['dateUniqKey'], dateQuery['end'])
        where = "%s AND %s" % (where1, where2)
        return where

    def getTable(self, tableName):
        ISTEST = self.ISTEST
        if ISTEST:
            tableName = "test_%s" % tableName
        if not self.outputHeaders.get(tableName, None):
            # mlog(self.outputHeaders)
            mlog("unkonw tableName1", tableName)
        mlog("opr------------", tableName)
        try:
            table = self.dsData.get_table(tableName)
        except OpenDSException as _:
            headers = self.outputHeaders[tableName]["headers"]
            schema = list(map(lambda header: {
                "name": header[0], "type": header[1]}, headers))
            table = self.dsData.create_table(tableName, schema, uniq_key=None)
        return table

    def getTableId(self, tableName):
        return self.getTable(tableName).get_id()

    def deleteTable(self, tableName):
        ISTEST = self.ISTEST
        if ISTEST:
            tableName = "test_%s" % tableName
            # 只有test才能删除表，如果正式要删除表，需要把下面代码的和if平级
            if not self.outputHeaders.get(tableName, None):
                mlog("unkonw tableName2", tableName)
            try:
                self.dsData.delete_table(tableName)
                mlog("delete table success", tableName)
            except Exception as e:
                mlog("table not exists", tableName, e)
